import * as puppeteer from "puppeteer";
import {
    sanitizeText,
    openPage,
    openaiTranslateToEnglish,
    openaiClassifyIntent,
    openaiFindQuestions,
} from './scraper_tele2_common.js';
import fs from "fs";

const DEBUG = true;
const URL = 'https://www.tele2.lv/privatpersonam/tarifu-plani/arpus-latvijas/zvani-un-internets-celojot/'
const HOSTNAME = 'https://www.tele2.lv/';
const FILE_NAME = 'tele2_data.json';

let urlignore = [
    // 'https://www.tele2.lv/palidziba/',
    'https://www.tele2.lv/tele2-privatuma-politika/',
    'https://www.tele2.lv/blogs/',
    'https://www.tele2.lv/telefoni/',
    'https://www.tele2.lv/datori/',
    'https://www.tele2.lv/televizori/',
    'https://www.tele2.lv/monitori/',
    'https://www.tele2.lv/viedpaligi/',
    'https://www.tele2.lv/plansetes/',
    'https://www.tele2.lv/aksesuari/',
];

let questionAndAnswers = [];
let answers = [];
let visitedUrls = [];
let urlqueue = [URL];

const addToQuestionAndAnswers = async (question_lv, answer_lv, title_lv, url) => {
    answer_lv = sanitizeText(answer_lv);
    question_lv = sanitizeText(question_lv);
    console.log(`addToQuestionAndAnswers`);
    questionAndAnswers.push({
        question_lv: question_lv,
        answer_lv: answer_lv,
        title_lv: title_lv,
        answer_en: await openaiTranslateToEnglish(answer_lv),
        question_en: await openaiTranslateToEnglish(question_lv),
        class: await openaiClassifyIntent(answer_lv),
        reference: url
    });
    console.log(`questionAndAnswers: ${questionAndAnswers.length}`);
    fs.writeFileSync(FILE_NAME,
        JSON.stringify(questionAndAnswers, null, "\t"),
        {encoding:'utf8'}
    );
}


const scrapePage = async (page, url) => {
    // remove # and everything after it
    url = url.replace(/(#.*)/g, '');
    url = url.trim();
    let langCheck = new RegExp('\/(ru|en)(\/|$)'); // foreign lan check
    let is_non_latvian = langCheck.test(url);

    if(url.startsWith(HOSTNAME) && !is_non_latvian && !url.includes('?'))
    {
        await openPage(page, url);

        if (!visitedUrls.includes(url) && !urlignore.includes(url)) {
            visitedUrls.push(url);
            console.log(`scrapePage: ${url}`);
            try {

                let head_title = await page.$eval('head title', title => title.textContent);
                head_title = head_title.trim();

                let is_questions_found = false;

                // Run first for only tele2.lv/palidziba
                if (url.startsWith('https://www.tele2.lv/palidziba/')) {
                    let $_support_question = await page.$('.nfaq_atbilde h1');
                    let $_support_answer = await page.$('.nfaq_atbilde .answer');
                    if($_support_question && $_support_answer) {
                        console.log("Found client support question");
                        let text_support_question = await page.evaluate(el => el.textContent, $_support_question);
                        let text_support_answer = await page.evaluate(el => el.textContent, $_support_answer);
                        text_support_answer = sanitizeText(text_support_answer);
                        if (isValidText(text_support_answer)) {
                            if (!answers.includes(text_support_answer)) {
                                await addToQuestionAndAnswers(text_support_question, text_support_answer, head_title, url);
                                is_questions_found = true;
                            }
                        }
                    }
                }
                //Run second, uncomment tele2.lv/palidziba in urlignore
                // if (!is_questions_found) {
                //     console.log("No regular questions found");
                //     await page.waitForTimeout(5000);
                //     let remove_div_selectors = [
                //         '.product_list_block',
                //         '.header',
                //         '.nfaq_atbilde',
                //         '.opendoc_block.answer',
                //         '.akcijas_2022',
                //         '.usp2022wrap',
                //         '.faq2019',
                //         '#leads_2023',
                //         '.page-header',
                //         '#cuvo-form',
                //         '.page_bottom',
                //         '#cybot',
                //         '.mob-footer',
                //         '.soho_promo2022'
                //     ];
                //     for (let selector of remove_div_selectors) {
                //         await page.evaluate((sel) => {
                //             let elements = document.querySelectorAll(sel);
                //             if (elements !== null) {
                //                 for (let i = 0; i < elements.length; i++) {
                //                     elements[i].parentNode.removeChild(elements[i]);
                //                 }
                //                 console.log(`Found elements with selector ${sel}: ${elements.length} instances`)
                // // Visible in webpage console
                //             } else {
                //                 console.log(`No elements with selector ${sel}`)
                //             }
                //         }, selector);
                //     }
                //     let whole_page = await page.$('body');
                //     let plaintext = await page.evaluate(el => el.innerText.trim(), whole_page);
                //     plaintext = sanitizeText(plaintext);
                //     if (plaintext) {
                //         let chunks_json = await openaiFindQuestions(plaintext);
                //         let question_answer_pairs = [];
                //         try {
                //             question_answer_pairs = JSON.parse(chunks_json);
                //         } catch (e) {
                //             console.log(chunks_json);
                //             console.error(e);
                //             throw new Error(e);
                //         }
                //         console.log(question_answer_pairs.length);
                //         for (let pair of question_answer_pairs) {
                //             await addToQuestionAndAnswers(pair.question, pair.answer, head_title, url);
                //             is_questions_found = true;
                //         }
                //     }
                // }

                if (!is_questions_found) {
                    console.log(`No questions found`);
                }

            } catch (e) {
                console.log(`Error: ${e}`);
                console.log(e.stack);
            }
        } // end of scraping

        try {
            let links = await page.$$eval('a', links => links.map(link => link.href))
            // debugging page.$$eval('a', links => links.map(link => link.href)).then(result => console.log(result))

            console.log(`links: ${links.length}`)
            for (let link of links) {
                if (link && link !== url) {
                    //URL does not end with .pdf using regex
                    let regex = new RegExp(/\.[a-z]{3}$/);
                    if (!regex.test(link) &&
                        link.startsWith(HOSTNAME) &&
                        !visitedUrls.includes(link) &&
                        !urlqueue.includes(link) &&
                        urlignore.includes(link)
                    ) {
                        urlqueue.push(link);
                    }
                }
            }
        } catch (e) {
            //console.log(`Error: ${e}`);

        }
    }
};

const main = async () => {

    //if file exists, read it
    if (fs.existsSync(FILE_NAME)) {
        let rawdata = fs.readFileSync(FILE_NAME);
        questionAndAnswers = JSON.parse(rawdata);

        //add urls to visitedUrls
        for (let item of questionAndAnswers) {
            visitedUrls.push(item.reference);
            answers.push(item.answer_lv);
        }
    }

    const browser = await puppeteer.launch({
        headless: false,
        slowMo: DEBUG,
        defaultViewport: null,
        args: [
            '--start-maximized' //can also use '--start-fullscreen' and those two ways work just for the start
        ],
        timeout: 60000
    });

    const page = await browser.newPage();
    while (urlqueue.length > 0) {
        await scrapePage(page, urlqueue[0]);
        urlqueue.shift();
    }
    console.log(`all finished questionAndAnswers length: ${questionAndAnswers.length}`)
    process.exit(0);
}

main();
